{-
This parser is used to parse syntax grammar files
and output internal representation of syntaxes.
-}

module Parsing.SyntaxParser where

import Parsing.Syntax
import Parsing.MainParsing

import Process.Result


{--------------------------------------------------------------------
--	Parser for a syntax definition file
--------------------------------------------------------------------}

readSyntax :: TextParser
readSyntax = parseText (syntaxParser syntax_def)

	
syntax_def :: SyntaxGrammar
syntax_def = SyntaxGrammar {
	startSyntaxes = [
		"syntax-def",
		"token-def",
		"skipped",
		"delimiters",
		"start-syntaxes"] ,

	skipped = [
		" ",
		"\t"
		],

	delimiters = [
		"(",")","?","*","+"],
	
	rules = [
		("syntax-def",     Syntax syntaxDef),

		("token-def",      Syntax tokenDef),

		("skipped",        Token Bounded "skip"),
		("delimiters",     Token Bounded "delimiter"),
		("start-syntaxes", Token Bounded "start-syntaxes"),
		
		("syntax-rule",    Syntax syntaxRule),
		("sub-syntax-rule",    Syntax subSyntaxRule),

		("rule-name",      Token Bounded "[a-zA-Z_][a-zA-Z0-9_\\-]*"),
		("modifier",       Token Greedy "[\\?\\*\\+]"),
		("anon-token",     Token Greedy "\\\".+\\\""),
		("regex",          Token Greedy ".+"),
		("char",           Token Bounded ".")
		],

	bodies = [
		("syntax-def", ["syntax-rule"]),
		("token-def", ["regex"]),
		("skipped", ["regex"]),
		("delimiters", ["char"]),
		("start-syntaxes", ["rule-name"])]
	}

syntaxDef
	= Sequence [
		DirectToken "syntax",
		Rule "rule-name"]

tokenDef
	= Sequence [
		DirectToken "token",
		Maybe (DirectToken "greedy"),
		Rule "rule-name"]

syntaxRule
	= OneOrMore (
		Sequence [
			Either [
				Rule "sub-syntax-rule",
				Rule "rule-name",
				Rule "anon-token"
				],
			Maybe (Rule "modifier")
			])

subSyntaxRule
	= Sequence [
		DirectToken "(",
		Rule "syntax-rule",
		DirectToken ")"]


--toAst :: ParseTree -> SyntaxGrammar
--toAst tree = syntaxDef





